#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Grammars {
  public class Symbol : ISymbol {
    public string Name { get; private set; }
    public IEnumerable<IAttribute> Attributes { get; private set; }
    public Symbol(string name, IEnumerable<IAttribute> attributes = null) {
      this.Name = name;
      if (attributes != null) this.Attributes = attributes;
      else this.Attributes = Enumerable.Empty<IAttribute>();
    }

    public string GetAttributeString() {
      if (!Attributes.Any()) return string.Empty;
      return Attributes.Skip(1).Aggregate(Attributes.First().ToString(), (str, a) => str + ", " + a.ToString());
    }

    public override bool Equals(object obj) {
      if (obj == null) return false;
      if (obj == this) return true;
      var other = obj as Symbol;
      if (other == null) return false;
      return this.Name == other.Name; // compare names only
    }

    public override int GetHashCode() {
      return Name.GetHashCode();
    }

    public override string ToString() {
      return Name + "<" + GetAttributeString() + ">";
    }
  }
}
